@@ -18,6 +18,7 @@ from toucan_connectors.common import (
     convert_to_qmark_paramstyle,
 )
 from toucan_connectors.toucan_connector import (
+    Category,
     DataSlice,
     ToucanConnector,
     ToucanDataSource,
@@ -81,6 +82,11 @@ class SnowflakeDataSource(ToucanDataSource):
 
 
 class AuthenticationMethod(str, Enum):
+    PLAIN: str = 'Snowflake (ID + Password)'
+    OAUTH: str = 'oAuth'
+
+
+class AuthenticationMethodValue(str, Enum):
     PLAIN: str = 'snowflake'
     OAUTH: str = 'oauth'
 
@@ -115,8 +121,11 @@ class SnowflakeConnector(ToucanConnector):
 
     user: str = Field(..., description='Your login username')
     password: SecretStr = Field(None, description='Your login password')
-    oauth_token: str = Field(None, description='Your oauth token')
-    oauth_args: dict = Field(None, description='Named arguments for an OIDC auth')
+    token_endpoint: Optional[str] = Field(None, description='The token endpoint')
+    token_endpoint_content_type: str = Field(
+        'application/json',
+        description='The content type to use when requesting the token endpoint',
+    )
 
     role: str = Field(
         None,
@@ -127,6 +136,7 @@ class SnowflakeConnector(ToucanConnector):
     default_warehouse: str = Field(
         None, description='The default warehouse that shall be used for any data source'
     )
+    category: Category = Field(Category.SNOWFLAKE, title='category', **{'ui': {'checkbox': False}})
 
     class Config:
         @staticmethod
@@ -138,8 +148,8 @@ class SnowflakeConnector(ToucanConnector):
                 'authentication_method',
                 'user',
                 'password',
-                'oauth_token',
-                'oauth_args',
+                'token_endpoint',
+                'token_endpoint_content_type',
                 'role',
                 'default_warehouse',
                 'retry_policy',
@@ -151,12 +161,31 @@ class SnowflakeConnector(ToucanConnector):
 
     @staticmethod
     def _get_status_details(index: int, status: Optional[bool]):
-        checks = ['Connection to Snowflake', 'Warehouse exists']
+        checks = ['Connection to Snowflake', 'Default warehouse exists']
         ok_checks = [(check, True) for i, check in enumerate(checks) if i < index]
         new_check = (checks[index], status)
         not_validated_checks = [(check, None) for i, check in enumerate(checks) if i > index]
         return ok_checks + [new_check] + not_validated_checks
 
+    @property
+    def access_token(self) -> Optional[str]:
+        return self.user_tokens_keeper and self.user_tokens_keeper.access_token.get_secret_value()
+
+    @property
+    def refresh_token(self) -> Optional[str]:
+        return self.user_tokens_keeper and self.user_tokens_keeper.refresh_token.get_secret_value()
+
+    @property
+    def client_id(self) -> Optional[str]:
+        return self.sso_credentials_keeper and self.sso_credentials_keeper.client_id
+
+    @property
+    def client_secret(self) -> Optional[str]:
+        return (
+            self.sso_credentials_keeper
+            and self.sso_credentials_keeper.client_secret.get_secret_value()
+        )
+
     def get_status(self) -> ConnectorStatus:
         try:
             with self.connect(login_timeout=5) as connection:
@@ -215,14 +244,16 @@ class SnowflakeConnector(ToucanConnector):
         if not self.authentication_method:
             # Default to User/Password authentication method if the parameter
             # was not set when the connector was created
-            res['authenticator'] = AuthenticationMethod.PLAIN
+            res['authenticator'] = AuthenticationMethodValue.PLAIN
 
         if res['authenticator'] == AuthenticationMethod.PLAIN and self.password:
+            res['authenticator'] = AuthenticationMethodValue.PLAIN
             res['password'] = self.password.get_secret_value()
 
         if self.authentication_method == AuthenticationMethod.OAUTH:
-            if self.oauth_token is not None:
-                res['token'] = Template(self.oauth_token).render()
+            if self.access_token is not None:
+                res['token'] = self.access_token
+            res['authenticator'] = AuthenticationMethodValue.OAUTH
 
         if self.role != '':
             res['role'] = self.role
@@ -231,40 +262,34 @@ class SnowflakeConnector(ToucanConnector):
 
     def _refresh_oauth_token(self):
         """Regenerates an oauth token if configuration was provided and if the given token has expired."""
-        if 'token_endpoint' in self.oauth_args and 'refresh_token' in self.oauth_args:
+        if self.token_endpoint and self.refresh_token:
             access_token = jwt.decode(
-                Template(self.oauth_token).render(),
+                self.access_token,
                 verify=False,
                 options={'verify_signature': False},
             )
             if datetime.fromtimestamp(access_token['exp']) < datetime.now():
-                content_type = 'application/json'
-                # Content-Type may need to be overridden
-                if 'content_type' in self.oauth_args:
-                    content_type = self.oauth_args['content_type']
-
                 res = requests.post(
-                    Template(self.oauth_args['token_endpoint']).render(),
+                    self.token_endpoint,
                     data={
                         'grant_type': 'refresh_token',
-                        'client_id': Template(self.oauth_args['client_id']).render(),
-                        'client_secret': Template(self.oauth_args['client_secret']).render(),
-                        'refresh_token': Template(self.oauth_args['refresh_token']).render(),
+                        'client_id': self.client_id,
+                        'client_secret': self.client_secret,
+                        'refresh_token': self.refresh_token,
                     },
-                    headers={'Content-Type': content_type},
+                    headers={'Content-Type': self.token_endpoint_content_type},
                 )
+                res.raise_for_status()
 
-                # Check if the request has a status_code equals to 200 and raise if not
-                # https://github.com/psf/requests/blob/master/requests/models.py#L936-L943
-                if not res.ok:  # pragma: no cover
-                    res.raise_for_status()
-
-                self.oauth_token = res.json().get('access_token')
+                self.user_tokens_keeper.update_tokens(
+                    access_token=res.json().get('access_token'),
+                    refresh_token=res.json().get('refresh_token'),
+                )
 
     def connect(self, **kwargs) -> snowflake.connector.SnowflakeConnection:
         # This needs to be set before we connect
         snowflake.connector.paramstyle = 'qmark'
-        if self.oauth_args and self.authentication_method == AuthenticationMethod.OAUTH:
+        if self.authentication_method == AuthenticationMethod.OAUTH:
             self._refresh_oauth_token()
         connection_params = self.get_connection_params()
 
@@ -310,6 +335,7 @@ class SnowflakeConnector(ToucanConnector):
                 values = pd.DataFrame.from_dict(query_res.fetchall())
             else:
                 values = pd.DataFrame.from_dict(query_res.fetchmany(max_rows))
+
         return values
 
     def _fetch_data(
@@ -342,4 +368,5 @@ class SnowflakeConnector(ToucanConnector):
 
         rows_to_fetch = offset + limit
         df = self._fetch_data(data_source, rows_to_fetch)
-        return df[offset:]
+
+        return DataSlice(df[offset:], len(df[offset:]))
