@@ -1,3 +1,5 @@
+from typing import List
+
 import pandas as pd
 import snowflake as pysnowflake
 from pydantic import Field, SecretStr
@@ -11,11 +13,11 @@ from toucan_connectors.snowflake.snowflake_connector import (
     AuthenticationMethod,
     SnowflakeDataSource,
 )
-from toucan_connectors.toucan_connector import ToucanConnector
+from toucan_connectors.toucan_connector import Category, ToucanConnector
 
 
 class SnowflakeoAuth2DataSource(SnowflakeDataSource):
-    """Nothing reimplemented from inherited class"""
+    """ """
 
 
 class SnowflakeoAuth2Connector(ToucanConnector):
@@ -59,6 +61,7 @@ class SnowflakeoAuth2Connector(ToucanConnector):
     default_warehouse: str = Field(
         ..., description='The default warehouse that shall be used for any data source'
     )
+    category: Category = Field(Category.SNOWFLAKE, title='category', **{'ui': {'checkbox': False}})
 
     def __init__(self, **kwargs):
         super().__init__(**{k: v for k, v in kwargs.items() if k != 'secrets_keeper'})
@@ -83,21 +86,30 @@ class SnowflakeoAuth2Connector(ToucanConnector):
     def retrieve_tokens(self, authorization_response: str):
         return self.__dict__['_oauth2_connector'].retrieve_tokens(authorization_response)
 
+    def _get_warehouses(self) -> List[str]:
+        with self.connect() as connection:
+            return [
+                warehouse['name']
+                for warehouse in connection.cursor(DictCursor).execute('SHOW WAREHOUSES').fetchall()
+                if 'name' in warehouse
+            ]
+
     def get_access_token(self):
         return self.__dict__['_oauth2_connector'].get_access_token()
 
-    def connect(self, database: str, warehouse: str) -> pysnowflake.connector.SnowflakeConnection:
+    def connect(self, database=None, warehouse=None) -> pysnowflake.connector.SnowflakeConnection:
         connection_params = {
             'account': self.account,
             'authenticator': AuthenticationMethod.OAUTH,
             'application': 'ToucanToco',
             'token': self.get_access_token(),
+            'role': self.role if self.role else '',
         }
         return pysnowflake.connector.connect(
             **connection_params, database=database, warehouse=warehouse
         )
 
-    def _retrieve_data(self, data_source: SnowflakeDataSource) -> pd.DataFrame:
+    def _retrieve_data(self, data_source: SnowflakeoAuth2DataSource) -> pd.DataFrame:
         connection = self.connect(
             database=data_source.database,
             warehouse=data_source.warehouse,
